@@ -21,18 +21,22 @@
 
 #ifdef CONFIG_APPLET_CLOCK
 
-YPixmap *PixNum[10] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-YPixmap *PixSpace = 0;
-YPixmap *PixColon = 0;
-YPixmap *PixSlash = 0;
-YPixmap *PixA = 0;
-YPixmap *PixP = 0;
-YPixmap *PixM = 0;
-YPixmap *PixDot = 0;
+ref<YPixmap> PixNum[10];
+ref<YPixmap> PixSpace;
+ref<YPixmap> PixColon;
+ref<YPixmap> PixSlash;
+ref<YPixmap> PixA;
+ref<YPixmap> PixP;
+ref<YPixmap> PixM;
+ref<YPixmap> PixDot;
+ref<YPixmap> PixPercent;
 
 YColor *YClock::clockBg = 0;
 YColor *YClock::clockFg = 0;
-YFont *YClock::clockFont = 0;
+ref<YFont> YClock::clockFont;
+
+extern ref<YPixmap> taskbackPixmap;
+static YColor *taskBarBg = 0;
 
 inline char const * strTimeFmt(struct tm const & t) {
     return (fmtTimeAlt && (t.tm_sec & 1) ? fmtTimeAlt : fmtTime);
@@ -43,11 +47,15 @@ YClock::YClock(YWindow *aParent): YWindow(aParent) {
         clockBg = new YColor(clrClock);
     if (clockFg == 0)
         clockFg = new YColor(clrClockText);
-    if (clockFont == 0)
-        clockFont = YFont::getFont(clockFontName);
+    if (clockFont == null)
+        clockFont = YFont::getFont(XFA(clockFontName));
+    if (taskBarBg == 0) {
+        taskBarBg = new YColor(clrDefaultTaskBar);
+    }
 
     clockUTC = false;
     toolTipUTC = false;
+    transparent = -1;
 
     clockTimer = new YTimer(1000);
     clockTimer->setTimerListener(this);
@@ -143,21 +151,25 @@ void YClock::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
+#ifdef DEBUG
 static int countEvents = 0;
 extern int xeventcount;
+#endif
 
 void YClock::handleClick(const XButtonEvent &up, int count) {
     if (up.button == 1) {
         if (clockCommand && clockCommand[0] &&
 	   (taskBarLaunchOnSingleClick ? count == 1 : !(count % 2)))
-	    wmapp->runCommandOnce(clockClassHint, clockCommand);
+            wmapp->runCommandOnce(clockClassHint, clockCommand);
+#ifdef DEBUG
     } else if (up.button == 2) {
         if ((count % 2) == 0)
             countEvents = !countEvents;
+#endif
     }
 }
 
-void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+void YClock::paint(Graphics &g, const YRect &/*r*/) {
     int x = width();
     char s[64];
     time_t newTime = time(NULL);
@@ -169,20 +181,44 @@ void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, un
     else
         t = localtime(&newTime);
 
+#ifdef DEBUG
     if (countEvents)
         len = sprintf(s, "%d", xeventcount);
     else
+#endif
         len = strftime(s, sizeof(s), strTimeFmt(*t), t);
 
+    
+    //clean backgroung first, so that it is possible
+    //to use transparent lcd pixmaps
+    if (hasTransparency()) {
+#ifdef CONFIG_GRADIENTS
+        ref<YPixbuf> gradient(parent()->getGradient());
+    
+        if (gradient != null)
+            g.copyPixbuf(*gradient, this->x(), this->y(),
+                         width(), height(), 0, 0);
+        else 
+#endif
+        if (taskbackPixmap != null) {
+            g.fillPixmap(taskbackPixmap, 0, 0,
+                         width(), height(), this->x(), this->y());
+        }
+        else {
+            g.setColor(taskBarBg);
+            g.fillRect(0, 0, width(), height());
+        }
+    }
+
     if (prettyClock) {
         i = len - 1;
         for (i = len - 1; x >= 0; i--) {
-            YPixmap *p;
+            ref<YPixmap> p;
             if (i >= 0)
                 p = getPixmap(s[i]);
             else
                 p = PixSpace;
-            if (p) {
+            if (p != null) {
                 x -= p->width();
                 g.drawPixmap(p, x, 0);
             } else if (i < 0) {
@@ -198,19 +234,7 @@ void YClock::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, un
 	if (clockBg) {
 	    g.setColor(clockBg);
             g.fillRect(0, 0, width(), height());
-	} else {
-#ifdef CONFIG_GRADIENTS
-	    class YPixbuf * gradient(parent()->getGradient());
-
-	    if (gradient)
-		g.copyPixbuf(*gradient, this->x(), this->y(),
-			     width(), height(), 0, 0);
-	    else 
-#endif	    
-	    if (taskbackPixmap)
-	        g.fillPixmap(taskbackPixmap, 0, 0,
-			     width(), height(), this->x(), this->y());
-	}
+        }
 
         g.setColor(clockFg);
         g.setFont(clockFont);
@@ -228,8 +252,8 @@ bool YClock::handleTimer(YTimer *t) {
     return true;
 }
 
-YPixmap *YClock::getPixmap(char c) {
-    YPixmap *pix = 0;
+ref<YPixmap> YClock::getPixmap(char c) {
+    ref<YPixmap> pix;
     switch (c) {
     case '0':
     case '1':
@@ -278,11 +302,29 @@ int YClock::calcWidth(const char *s, int count) {
         int len = 0;
 
         while (count--) {
-            YPixmap *p = getPixmap(*s++);
-            if (p)
+            ref<YPixmap> p = getPixmap(*s++);
+            if (p != null)
                 len += p->width();
         }
         return len;
     }
 }
+
+bool YClock::hasTransparency() {
+    if (transparent == 0)
+        return false;
+    else if (transparent == 1)
+        return true;
+    if (!prettyClock) {
+        transparent = 1;
+        return true;
+    }
+    ref<YPixmap> p = getPixmap('0');
+    if (p != null && p->mask()) {
+        transparent = 1;
+        return true;
+    }
+    transparent = 0;
+    return false;
+}
 #endif
