@@ -4,6 +4,16 @@
  * Copyright (C) 1998-2001 Marko Macek
  *
  * CPU Status
+ *
+ * KNOWN BUGS:
+ * 1. On startup, the second bar shows always zero, and the first bar shows
+ *    total cpu info from bootup time (maybe this can be also considered as
+ *    a feature) -stibor-
+ *
+ * 2. There are no checks for overflows: when sys, user, nice or idle time
+ *    in /proc/stat will exceed usigned long (256^4), CPU monitor will
+ *    probably show nonsenses. (but it is at least after 500 days of uptime)
+ *    -stibor-
  */
 #include "config.h"
 
@@ -19,18 +29,36 @@
 //#include <linux/kernel.h>
 #include <sys/sysinfo.h>
 #endif
-
+#if defined(sun) && defined(SVR4)
+#include <sys/loadavg.h>
+#endif
 #ifdef HAVE_KSTAT_H
+#include <sys/resource.h>
 #include <kstat.h>
+#include <sys/sysinfo.h>
+#endif
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#endif
+#ifdef HAVE_UVM_UVM_PARAM_H
+#include <uvm/uvm_param.h>
+#endif
+#ifdef HAVE_SYS_DKSTAT_H
+#include <sys/dkstat.h>
+#endif
+#ifdef HAVE_SCHED_H
+#include <sched.h>
 #endif
 
 #include "intl.h"
 
-#if (defined(linux) || defined(HAVE_KSTAT_H))
+#if (defined(linux) || defined(HAVE_KSTAT_H)) || defined(HAVE_SYSCTL_CP_TIME)
 
-#define UPDATE_INTERVAL 500
-
-extern YPixmap *taskbackPixmap;
+extern ref<YPixmap> taskbackPixmap;
 
 CPUStatus::CPUStatus(YWindow *aParent): YWindow(aParent) {
     cpu = new int *[taskBarCPUSamples];
@@ -38,7 +66,7 @@ CPUStatus::CPUStatus(YWindow *aParent): YWindow(aParent) {
     for (int a(0); a < taskBarCPUSamples; a++)
         cpu[a] = new int[IWM_STATES];
 
-    fUpdateTimer = new YTimer(UPDATE_INTERVAL);
+    fUpdateTimer = new YTimer(taskBarCPUDelay);
     if (fUpdateTimer) {
         fUpdateTimer->setTimerListener(this);
         fUpdateTimer->startTimer();
@@ -47,15 +75,21 @@ CPUStatus::CPUStatus(YWindow *aParent): YWindow(aParent) {
     color[IWM_USER] = new YColor(clrCpuUser);
     color[IWM_NICE] = new YColor(clrCpuNice);
     color[IWM_SYS]  = new YColor(clrCpuSys);
+    color[IWM_INTR] = new YColor(clrCpuIntr);
+    color[IWM_IOWAIT] = new YColor(clrCpuIoWait);
+    color[IWM_SOFTIRQ] = new YColor(clrCpuSoftIrq);
     color[IWM_IDLE] = *clrCpuIdle
     		    ? new YColor(clrCpuIdle) : NULL;
-
-    for (int i(0); i < taskBarCPUSamples; i++) {
-        cpu[i][IWM_USER] = cpu[i][IWM_NICE] = cpu[i][IWM_SYS] = 0;
+    for (int i = 0; i < taskBarCPUSamples; i++) {
+        cpu[i][IWM_USER] = cpu[i][IWM_NICE] =
+        cpu[i][IWM_SYS] = cpu[i][IWM_INTR] =
+	cpu[i][IWM_IOWAIT] = cpu[i][IWM_SOFTIRQ] = 0;
         cpu[i][IWM_IDLE] = 1;
     }
     setSize(taskBarCPUSamples, 20);
-    last_cpu[IWM_USER] = last_cpu[IWM_NICE] = last_cpu[IWM_SYS] = last_cpu[IWM_IDLE] = 0;
+    last_cpu[IWM_USER] = last_cpu[IWM_NICE] = last_cpu[IWM_SYS] =
+    last_cpu[IWM_IDLE] = last_cpu[IWM_INTR] =
+    last_cpu[IWM_IOWAIT] = last_cpu[IWM_SOFTIRQ] = 0;
     getStatus();
     updateStatus();
     updateToolTip();
@@ -70,59 +104,89 @@ CPUStatus::~CPUStatus() {
     delete color[IWM_NICE]; color[IWM_NICE] = 0;
     delete color[IWM_SYS];  color[IWM_SYS]  = 0;
     delete color[IWM_IDLE]; color[IWM_IDLE] = 0;
+    delete color[IWM_INTR]; color[IWM_INTR] = 0;
+    delete color[IWM_IOWAIT]; color[IWM_IOWAIT] = 0;
+    delete color[IWM_SOFTIRQ]; color[IWM_SOFTIRQ] = 0;
 }
 
-void CPUStatus::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
-    int n, h = height();
+void CPUStatus::paint(Graphics &g, const YRect &/*r*/) {
+    int h = height();
 
     for (int i(0); i < taskBarCPUSamples; i++) {
         int user = cpu[i][IWM_USER];
         int nice = cpu[i][IWM_NICE];
         int sys = cpu[i][IWM_SYS];
         int idle = cpu[i][IWM_IDLE];
-        int total = user + sys + nice + idle;
+        int intr = cpu[i][IWM_INTR];
+	int iowait = cpu[i][IWM_IOWAIT];
+	int softirq = cpu[i][IWM_SOFTIRQ];
+        int total = user + sys + intr + nice + idle + iowait + softirq;
 
         int y = height() - 1;
 
-        if (total > 0) {
-            if (sys) {
-                n = (h * (total - sys)) / total; // check rounding
-                if (n >= y) n = y;
+        if (total > 1) { /* better show 0 % CPU than nonsense on startup */
+            int intrbar, sysbar, nicebar, userbar, iowaitbar, softirqbar;
+            int round = total / h / 2;  /* compute also with rounding errs */
+
+            if ((intrbar = (h * (intr + round)) / total)) {
+                g.setColor(color[IWM_INTR]);
+                g.drawLine(i, y, i, y - (intrbar - 1));
+                y -= intrbar;
+            }
+            if ((softirqbar = (h * (softirq + round)) / total)) {
+                g.setColor(color[IWM_SOFTIRQ]);
+                g.drawLine(i, y, i, y - (softirqbar - 1));
+                y -= softirqbar;
+            }
+            iowaitbar = (h * (iowait + round)) / total;
+            if ((sysbar = (h * (sys + round)) / total)) {
                 g.setColor(color[IWM_SYS]);
-                g.drawLine(i, y, i, n);
-                y = n - 1;
+                g.drawLine(i, y, i, y - (sysbar - 1));
+                y -= sysbar;
             }
 
-            if (nice) {
-                n = (h * (total - sys - nice))/ total;
-                if (n >= y) n = y;
-                g.setColor(color[IWM_NICE]);
-                g.drawLine(i, y, i, n);
-                y = n - 1;
-            }
+            nicebar = (h * (nice + round)) / total;
 
-            if (user) {
-                n = (h * (total - sys - nice - user))/ total;
-                if (n >= y) n = y;
+            /* minor rounding errors are counted into user bar: */
+            if ((userbar = (h * ((sys + nice + user + intr + iowait + softirq) +
+							       round)) / total -
+                           (sysbar + nicebar + intrbar + iowaitbar + softirqbar)
+			   						      ))
+	    {
                 g.setColor(color[IWM_USER]);
-                g.drawLine(i, y, i, n);
-                y = n - 1;
+                g.drawLine(i, y, i, y - (userbar - 1));
+                y -= userbar;
             }
+
+            if (nicebar) {
+                g.setColor(color[IWM_NICE]);
+                g.drawLine(i, y, i, y - (nicebar - 1));
+                y -= nicebar;
+            }
+            if (iowaitbar) {
+                g.setColor(color[IWM_IOWAIT]);
+                g.drawLine(i, y, i, y - (iowaitbar - 1));
+                y -= iowaitbar;
+            }
+#if 0
+            msg(_("stat:\tuser = %i, nice = %i, sys = %i, idle = %i"), cpu[i][IWM_USER], cpu[i][IWM_NICE], cpu[i][IWM_SYS], cpu[i][IWM_IDLE]);
+            msg(_("bars:\tuser = %i, nice = %i, sys = %i (h = %i)\n"), userbar, nicebar, sysbar, h);
+#endif
         }
-        if (idle) {
+        if (y > 0) {
 	    if (color[IWM_IDLE]) {
 		g.setColor(color[IWM_IDLE]);
 		g.drawLine(i, 0, i, y);
             } else {
 #ifdef CONFIG_GRADIENTS
-		class YPixbuf * gradient(parent()->getGradient());
+		ref<YPixbuf> gradient = parent()->getGradient();
 
-		if (gradient)
+		if (gradient != null)
 		    g.copyPixbuf(*gradient,
 		    		 this->x() + i, this->y(), width(), y + 1, i, 0);
 		else 
 #endif		
-		if (taskbackPixmap)
+		if (taskbackPixmap != null)
 		    g.fillPixmap(taskbackPixmap,
 		    		 i, 0, width(), y + 1, this->x() + i, this->y());
 	    }
@@ -141,7 +205,7 @@ bool CPUStatus::handleTimer(YTimer *t) {
 
 void CPUStatus::updateToolTip() {
 #ifdef linux
-    char load[64];
+    char load[31];
     struct sysinfo sys;
     float l1, l5, l15;
 
@@ -149,9 +213,21 @@ void CPUStatus::updateToolTip() {
     l1 = (float)sys.loads[0] / 65536.0;
     l5 = (float)sys.loads[1] / 65536.0;
     l15 = (float)sys.loads[2] / 65536.0;
-    sprintf(load, _("CPU Load: %3.2f %3.2f %3.2f, %d processes."),
+    sprintf(load, "%3.2f %3.2f %3.2f, %d",
             l1, l5, l15, sys.procs);
-    setToolTip(load);
+    char *loadmsg = strJoin(_("CPU Load: "), load, _(" processes."), NULL);
+    setToolTip(loadmsg);
+    delete [] loadmsg;
+#elif defined HAVE_GETLOADAVG2
+    char load[31]; // enough for "CPU Load: 999.99 999.99 999.99\0"
+    double loadavg[3];
+    if (getloadavg(loadavg, 3) < 0)
+	return;
+    snprintf(load, sizeof(load), "CPU Load: %3.2f %3.2f %3.2f",
+            loadavg[0], loadavg[1], loadavg[2]);
+    char *loadmsg = strJoin(_("CPU Load: "), load, NULL);
+    setToolTip(loadmsg);
+    delete [] loadmsg;
 #endif
 }
 
@@ -169,6 +245,9 @@ void CPUStatus::updateStatus() {
         cpu[i - 1][IWM_NICE] = cpu[i][IWM_NICE];
         cpu[i - 1][IWM_SYS]  = cpu[i][IWM_SYS];
         cpu[i - 1][IWM_IDLE] = cpu[i][IWM_IDLE];
+        cpu[i - 1][IWM_INTR] = cpu[i][IWM_INTR];
+        cpu[i - 1][IWM_IOWAIT] = cpu[i][IWM_IOWAIT];
+        cpu[i - 1][IWM_SOFTIRQ] = cpu[i][IWM_SOFTIRQ];
     }
     getStatus(),
     repaint();
@@ -177,13 +256,16 @@ void CPUStatus::updateStatus() {
 void CPUStatus::getStatus() {
 #ifdef linux
     char *p, buf[128];
-    long cur[IWM_STATES];
-    int len, fd = open("/proc/stat", O_RDONLY);
+    unsigned long cur[IWM_STATES];
+    int len, s, fd = open("/proc/stat", O_RDONLY);
 
-    cpu[taskBarCPUSamples-1][IWM_USER] = 0;
-    cpu[taskBarCPUSamples-1][IWM_NICE] = 0;
-    cpu[taskBarCPUSamples-1][IWM_SYS] = 0;
-    cpu[taskBarCPUSamples-1][IWM_IDLE] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_USER] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_NICE] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_INTR] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_IOWAIT] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_SOFTIRQ] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_SYS] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_IDLE] = 0;
 
     if (fd == -1)
         return;
@@ -192,22 +274,45 @@ void CPUStatus::getStatus() {
         close(fd);
         return;
     }
+    close(fd);
     buf[len] = 0;
 
     p = buf;
     while (*p && (*p < '0' || *p > '9'))
         p++;
+    /* Linux 2.4:  cpu  3557 8 1052 7049
+     * Linux 2.6:  cpu  3537 44 1064 676229 7792 142 5
+     */
+    if ((s = strcspn(p, "\n")) <= 60)
+	strcpy(p + s, " 0 0 0\n");	/* Linux 2.4 */
 
-    for (int i = 0; i < 4; i++) {
-        cur[i] = strtoul(p, &p, 10);
-        cpu[taskBarCPUSamples-1][i] = cur[i] - last_cpu[i];
-        last_cpu[i] = cur[i];
+    int i = 0;
+    for (i = 0; i < 7; i++) {
+        int d = -1;
+
+	/*  linux/Documentation/filesystems/proc.txt: 1.8  */
+        switch (i) {
+        case 0: d = IWM_USER; break;
+        case 1: d = IWM_NICE; break;
+        case 2: d = IWM_SYS; break;
+        case 3: d = IWM_IDLE; break;
+        case 4: d = IWM_IOWAIT; break;
+        case 5: d = IWM_INTR; break;
+        case 6: d = IWM_SOFTIRQ; break;
+        }
+        cur[d] = strtoul(p, &p, 10);
+        cpu[taskBarCPUSamples - 1][d] = cur[d] - last_cpu[d];
+        last_cpu[d] = cur[d];
     }
-    close(fd);
 #if 0
-    msg(_("cpu: %d %d %d %d"),
-            cpu[taskBarCPUSamples-1][IWM_USER], cpu[taskBarCPUSamples-1][IWM_NICE],
-            cpu[taskBarCPUSamples-1][IWM_SYS],  cpu[taskBarCPUSamples-1][IDLE]);
+    msg(_("cpu: %d %d %d %d %d %d %d"),
+            cpu[taskBarCPUSamples-1][IWM_USER],
+	    cpu[taskBarCPUSamples-1][IWM_NICE],
+            cpu[taskBarCPUSamples-1][IWM_SYS],
+	    cpu[taskBarCPUSamples-1][IWM_IDLE],
+	    cpu[taskBarCPUSamples-1][IWM_IOWAIT],
+	    cpu[taskBarCPUSamples-1][IWM_INTR],
+	    cpu[taskBarCPUSamples-1][IWM_SOFTIRQ]);
 #endif
 #endif /* linux */
 #ifdef HAVE_KSTAT_H
@@ -350,7 +455,53 @@ void CPUStatus::getStatus() {
     cpu[taskBarCPUSamples-1][IWM_IDLE] = cp_pct[CPU_IDLE];
 
 #endif /* have_kstat_h */
+#if defined HAVE_SYSCTL_CP_TIME && defined CP_INTR
+#if defined __NetBSD__
+    typedef u_int64_t cp_time_t;
+#else
+    typedef long cp_time_t;
+#endif
+#if defined KERN_CPTIME
+    static int mib[] = { CTL_KERN, KERN_CPTIME };
+#elif defined KERN_CP_TIME
+    static int mib[] = { CTL_KERN, KERN_CP_TIME };
+#else
+    static int mib[] = { 0, 0 };
+#endif
+    
+    cpu[taskBarCPUSamples - 1][IWM_USER] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_NICE] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_SYS] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_INTR] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_IOWAIT] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_SOFTIRQ] = 0;
+    cpu[taskBarCPUSamples - 1][IWM_IDLE] = 0;
 
+
+    cp_time_t cp_time[CPUSTATES];
+    size_t len = sizeof( cp_time );
+#if defined HAVE_SYSCTLBYNAME
+    if (sysctlbyname("kern.cp_time", cp_time, &len, NULL, 0) < 0)
+	return;
+#else
+    if (sysctl(mib, 2, cp_time, &len, NULL, 0) < 0)
+	return;
+#endif
+
+    long cur[IWM_STATES];
+    cur[IWM_USER] = cp_time[CP_USER];
+    cur[IWM_NICE] = cp_time[CP_NICE];
+    cur[IWM_SYS] = cp_time[CP_SYS];
+    cur[IWM_INTR] = cp_time[CP_INTR];
+    cur[IWM_IOWAIT] = 0;
+    cur[IWM_SOFTIRQ] = 0;
+    cur[IWM_IDLE] = cp_time[CP_IDLE];
+
+    for (int i = 0; i < IWM_STATES; i++) {
+        cpu[taskBarCPUSamples - 1][i] = cur[i] - last_cpu[i];
+        last_cpu[i] = cur[i];
+    }
+#endif
 }
 #endif
 #endif
