@@ -8,19 +8,14 @@
 #ifndef NETSTATUS_H
 #define NETSTATUS_H
 
-#if defined(linux) || defined(__FreeBSD__)
 #ifdef CONFIG_APPLET_NET_STATUS
+#if defined(linux) || defined(__FreeBSD__)
 
 #define HAVE_NET_STATUS 1
 
 #include "ywindow.h"
 #include "ytimer.h"
 
-#include <sys/socket.h>
-
-#define NET_UPDATE_INTERVAL 500
-#define NET_SAMPLES 20
-
 class NetStatus: public YWindow, public YTimerListener {
 public:
     NetStatus(char const * netdev, YWindow *aParent = 0);
@@ -28,17 +23,14 @@ public:
 private:
     YColor *color[3];
     YTimer *fUpdateTimer;
-    int maxBytes;
 
-    long long ppp_in[NET_SAMPLES];
-    long long ppp_out[NET_SAMPLES];
-    long long ppp_tot[NET_SAMPLES];
+    long *ppp_in; /* long could be really enough for rate in B/s */
+    long *ppp_out;
 
     unsigned long long prev_ibytes, start_ibytes, cur_ibytes, offset_ibytes;
     unsigned long long prev_obytes, start_obytes, cur_obytes, offset_obytes;
 
     time_t start_time;
-
     struct timeval prev_time;
 
     bool wasUp;               // previous link status
@@ -50,18 +42,20 @@ private:
     // methods local to this class
     bool isUp();
     bool isUpIsdn();
-    void getCurrent(long long *in, long long *out, long long *tot);
+    void getCurrent(long *in, long *out);
     void updateStatus();
     void updateToolTip();
 
     // methods overloaded from superclasses
     virtual bool handleTimer(YTimer *t);
     virtual void handleClick(const XButtonEvent &up, int count);
-    virtual void paint(Graphics & g, int x, int y, unsigned int width, unsigned int height);
+    virtual void paint(Graphics & g, const YRect &r);
 };
 
+
+#else // linux || __FreeBSD__
+#undef CONFIG_APPLET_NET_STATUS
+#endif
 #endif // CONFIG_APPLET_NET_STATUS
 
-#endif // linux || __FreeBSD__
-
 #endif // NETSTATUS_H
