@@ -9,6 +9,8 @@
 #include "wmmgr.h"
 #include "wmapp.h"
 #include "wmframe.h"
+#include "yrect.h"
+#include "yicon.h"
 
 #include "intl.h"
 
@@ -27,18 +29,18 @@ YColor * WorkspaceButton::normalButtonFg(NULL);
 YColor * WorkspaceButton::activeButtonBg(NULL);
 YColor * WorkspaceButton::activeButtonFg(NULL);
 
-YFont * WorkspaceButton::normalButtonFont(NULL);
-YFont * WorkspaceButton::activeButtonFont(NULL);
+ref<YFont> WorkspaceButton::normalButtonFont;
+ref<YFont> WorkspaceButton::activeButtonFont;
 
-YPixmap *workspacebuttonPixmap(NULL);
-YPixmap *workspacebuttonactivePixmap(NULL);
+ref<YPixmap> workspacebuttonPixmap;
+ref<YPixmap> workspacebuttonactivePixmap;
 
 #ifdef CONFIG_GRADIENTS
-class YPixbuf *workspacebuttonPixbuf(NULL);
-class YPixbuf *workspacebuttonactivePixbuf(NULL);
+ref<YPixbuf> workspacebuttonPixbuf;
+ref<YPixbuf> workspacebuttonactivePixbuf;
 #endif
 
-WorkspaceButton::WorkspaceButton(long ws, YWindow *parent): YButton(parent, 0)
+WorkspaceButton::WorkspaceButton(long ws, YWindow *parent): ObjectButton(parent, (YAction *)0)
 {
     fWorkspace = ws;
     //setDND(true);
@@ -75,7 +77,7 @@ bool WorkspaceButton::handleTimer(YTimer *t) {
 void WorkspaceButton::actionPerformed(YAction */*action*/, unsigned int modifiers) {
     if (modifiers & ShiftMask) {
         manager->switchToWorkspace(fWorkspace, true);
-    } else if (modifiers & app->AltMask) {
+    } else if (modifiers & xapp->AltMask) {
         if (manager->getFocus())
             manager->getFocus()->wmOccupyWorkspace(fWorkspace);
     } else {
@@ -87,7 +89,7 @@ void WorkspaceButton::actionPerformed(YAction */*action*/, unsigned int modifier
 WorkspacesPane::WorkspacesPane(YWindow *parent): YWindow(parent) {
     long w;
 
-    if (workspaceCount > 1)
+    if (workspaceCount > 0)
         fWorkspaceButton = new WorkspaceButton *[workspaceCount];
     else
         fWorkspaceButton = 0;
@@ -95,19 +97,22 @@ WorkspacesPane::WorkspacesPane(YWindow *parent): YWindow(parent) {
     if (fWorkspaceButton) {
 	YResourcePaths paths("", false);
 
-        int ht = 0;
+        int ht = 24;
         int leftX = 0;
 
-        ht = parent->height();
+#warning "fixme"
+///        ht = parent->height();
 
         for (w = 0; w < workspaceCount; w++) {
             WorkspaceButton *wk = new WorkspaceButton(w, this);
             if (wk) {
-		YIcon::Image * image
+		ref<YIconImage> image
 		    (paths.loadImage("workspace/", workspaceNames[w]));
 
-		if (image) wk->setImage(image);
-                else wk->setText(workspaceNames[w]);
+                if (image != null)
+                    wk->setImage(image);
+                else
+                    wk->setText(workspaceNames[w]);
 		
 		char * wn(newstr(basename(workspaceNames[w])));
 		char * ext(strrchr(wn, '.'));
@@ -128,7 +133,7 @@ WorkspacesPane::WorkspacesPane(YWindow *parent): YWindow(parent) {
             YButton *wk = fWorkspaceButton[w];
             //leftX += 2;
             if (wk) {
-                wk->setGeometry(leftX, 0, wk->width(), ht);
+                wk->setGeometry(YRect(leftX, 0, wk->width(), ht));
                 wk->show();
                 leftX += wk->width();
             }
@@ -145,21 +150,36 @@ WorkspacesPane::~WorkspacesPane() {
     }
 }
 
+void WorkspacesPane::configure(const YRect &r, const bool resized) {
+    YWindow::configure(r, resized);
+
+    int ht = height();
+    int leftX = 0;
+    for (int w = 0; w < workspaceCount; w++) {
+        YButton *wk = fWorkspaceButton[w];
+        //leftX += 2;
+        if (wk) {
+            wk->setGeometry(YRect(leftX, 0, wk->width(), ht));
+            leftX += wk->width();
+        }
+    }
+}
+
 WorkspaceButton *WorkspacesPane::workspaceButton(long n) {
     return (fWorkspaceButton ? fWorkspaceButton[n] : NULL);
 }
 
-YFont * WorkspaceButton::getFont() {
+ref<YFont> WorkspaceButton::getFont() {
     return isPressed()
     	? *activeWorkspaceFontName
-	  ? activeButtonFont
+	  ? activeButtonFont != null
 	    ? activeButtonFont
-	    : activeButtonFont = YFont::getFont(activeWorkspaceFontName)
+	    : activeButtonFont = YFont::getFont(XFA(activeWorkspaceFontName))
 	  : YButton::getFont()
     	: *normalWorkspaceFontName
-	  ? normalButtonFont
+	  ? normalButtonFont != null
 	    ? normalButtonFont
-	    : normalButtonFont = YFont::getFont(normalWorkspaceFontName)
+	    : normalButtonFont = YFont::getFont(XFA(normalWorkspaceFontName))
 	  : YButton::getFont();
 }
 
