@@ -12,7 +12,10 @@
 #include "wmaction.h"
 #include "wmframe.h"
 #include "wmtitle.h"
-#include "yapp.h"
+#include "yxapp.h"
+#include "yicon.h"
+#include "yprefs.h"
+#include "prefs.h"
 
 static YColor *titleButtonBg = 0;
 static YColor *titleButtonFg = 0;
@@ -22,7 +25,7 @@ extern YColor *activeTitleBarBg;
 extern YColor *inactiveTitleBarBg;
 
 #ifdef CONFIG_LOOK_PIXMAP
-YPixmap *menuButton[2] = { 0, 0 };
+ref<YPixmap> menuButton[2];
 #endif
 
 YFrameButton::YFrameButton(YWindow *parent,
@@ -43,7 +46,7 @@ YFrameButton::YFrameButton(YWindow *parent,
 
 
     int w = 18, h = 18;
-    if (minimizePixmap[0]) {
+    if (minimizePixmap[0] != null) {
         w = minimizePixmap[0]->width();
         h = minimizePixmap[0]->height();
     }
@@ -107,9 +110,9 @@ void YFrameButton::handleClick(const XButtonEvent &up, int count) {
             setArmed(false, false);
             getFrame()->wmClose();
         }
-    } else if (up.button == 3 && (KEY_MODMASK(up.state) & (app->AltMask)) == 0) {
+    } else if (up.button == 3 && (KEY_MODMASK(up.state) & (xapp->AltMask)) == 0) {
         if (!isPopupActive())
-            getFrame()->popupSystemMenu(up.x_root, up.y_root, -1, -1,
+            getFrame()->popupSystemMenu(this, up.x_root, up.y_root,
                                         YPopupWindow::pfCanFlipVertical |
                                         YPopupWindow::pfCanFlipHorizontal);
     }
@@ -144,7 +147,7 @@ void YFrameButton::actionPerformed(YAction * /*action*/, unsigned int modifiers)
         getFrame()->actionPerformed(fAction, modifiers);
 }
 
-YPixmap *YFrameButton::getImage(int pn) const {
+ref<YPixmap> YFrameButton::getImage(int pn) const {
     if (fAction == actionMaximize)
         return maximizePixmap[pn];
     else if (fAction == actionMinimize)
@@ -167,26 +170,38 @@ YPixmap *YFrameButton::getImage(int pn) const {
 	return menuButton[pn];
 #endif	
     else
-	return NULL;
+	return null;
 }
 
-void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int ) {
+void YFrameButton::paint(Graphics &g, const YRect &/*r*/) {
     int xPos = 1, yPos = 1;
-    const int pn((wmLook == lookPixmap || wmLook == lookMetal || 
-    		  wmLook == lookGtk) && getFrame()->focused() ? 1 : 0);
+    int pn = (wmLook == lookPixmap || wmLook == lookMetal ||
+              wmLook == lookGtk) && getFrame()->focused() ? 1 : 0;
     const bool armed(isArmed());
 
     g.setColor(titleButtonBg);
 
+    if (fOver && rolloverTitleButtons) {
+        if (pn == 1) {
+            pn = 2;
+        } else {
+            pn = 0;
+        }
+    }
+
+
+
 #ifdef LITE
-    YIcon::Image * icon(NULL);
+    ref<YIconImage> icon;
 #else			
-    YIcon::Image * icon(fAction == 0 && getFrame()->clientIcon() ?
-			getFrame()->clientIcon()->small() : NULL);
+    ref<YIconImage> icon =
+        (fAction == 0 && getFrame()->clientIcon()) ?
+        getFrame()->clientIcon()->small() : null;
 #endif
 	    
-    YPixmap *pixmap((wmLook == lookPixmap || wmLook == lookMetal || 
-    		     wmLook == lookGtk) || fAction ? getImage(pn) : 0);
+    ref<YPixmap> pixmap =
+        ((wmLook == lookPixmap || wmLook == lookMetal ||
+         wmLook == lookGtk) || fAction) ? getImage(pn) : null;
 
     switch (wmLook) {
 #ifdef CONFIG_LOOK_WARP4
@@ -199,13 +214,13 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
 
             g.fillRect(1, 1, width() - 2, height() - 2);
 
-            if (icon && showFrameIcon)
+            if (icon != null && showFrameIcon)
                 g.drawImage(icon, (width() - icon->width()) / 2,
 				  (height() - icon->height()) / 2);
         } else {
             g.fillRect(0, 0, width(), height());
 
-            if (pixmap)
+            if (pixmap != null)
 		g.copyPixmap(pixmap, 0, armed ? 20 : 0,
 			     pixmap->width(), pixmap->height() / 2,
 			     (width() - pixmap->width()) / 2,
@@ -248,11 +263,11 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
         if (fAction == 0) {
             g.fillRect(xPos, yPos, w, h);
 
-            if (icon && showFrameIcon)
+            if (icon != null && showFrameIcon)
                 g.drawImage(icon, xPos + (w - icon->width()) / 2,
 				  yPos + (h - icon->height()) / 2);
         } else {
-            if (pixmap)
+            if (pixmap != null)
                 g.drawCenteredPixmap(xPos, yPos, w, h, pixmap);
         }
 
@@ -271,7 +286,7 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
 
             g.fillRect(0, 0, width(), height());
 
-            if (icon && showFrameIcon)
+            if (icon != null && showFrameIcon)
                 g.drawImage(icon, (width() - icon->width()) / 2,
 				  (height() - icon->height()) / 2);
         } else {
@@ -280,7 +295,7 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
             if (armed)
                 xPos = yPos = 2;
 
-            if (pixmap)
+            if (pixmap != null)
                 g.drawCenteredPixmap(xPos, yPos, width() - 3, height() - 3,
                                      pixmap);
         }
@@ -290,13 +305,13 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
     case lookPixmap:
     case lookMetal:
     case lookGtk:
-	if (pixmap) {
+	if (pixmap != null) {
 	    int const h(pixmap->height() / 2);
 	    g.copyPixmap(pixmap, 0, armed ? h : 0, pixmap->width(), h, 0, 0);
 	} else
 	   g.fillRect(0, 0, width(), height());
 
-	if (fAction == 0 && icon && showFrameIcon)
+	if (fAction == 0 && icon != null && showFrameIcon)
 	    g.drawImage(icon, ((int)width() - (int)icon->width()) / 2,
 			      ((int)height() - (int)icon->height()) / 2);
 
@@ -308,5 +323,5 @@ void YFrameButton::paint(Graphics &g, int , int , unsigned int , unsigned int )
 }
 
 
-void YFrameButton::paintFocus(Graphics &/*g*/, int /*x*/, int /*y*/, unsigned int /*w*/, unsigned int /*h*/) {
+void YFrameButton::paintFocus(Graphics &/*g*/, const YRect &/*r*/) {
 }
