@@ -11,19 +11,22 @@
 #include "wmminiicon.h"
 
 #include "wmframe.h"
-#include "yapp.h"
+#include "yxapp.h"
+#include "yrect.h"
+#include "yicon.h"
+#include "prefs.h"
 
 #include <string.h>
 
-static YFont *minimizedWindowFont = 0;
+static ref<YFont> minimizedWindowFont;
 static YColor *normalMinimizedWindowBg = 0;
 static YColor *normalMinimizedWindowFg = 0;
 static YColor *activeMinimizedWindowBg = 0;
 static YColor *activeMinimizedWindowFg = 0;
 
 MiniIcon::MiniIcon(YWindow *aParent, YFrameWindow *frame): YWindow(aParent) {
-    if (minimizedWindowFont == 0)
-        minimizedWindowFont = YFont::getFont(minimizedWindowFontName);
+    if (minimizedWindowFont == null)
+        minimizedWindowFont = YFont::getFont(XFA(minimizedWindowFontName));
     if (normalMinimizedWindowBg == 0)
         normalMinimizedWindowBg = new YColor(clrNormalMinimizedWindow);
     if (normalMinimizedWindowFg == 0)
@@ -35,14 +38,13 @@ MiniIcon::MiniIcon(YWindow *aParent, YFrameWindow *frame): YWindow(aParent) {
 
     fFrame = frame;
     selected = 0;
-    setGeometry(0, 0, 120, 24);
+    setGeometry(YRect(0, 0, 120, 24));
 }
 
 MiniIcon::~MiniIcon() {
 }
 
-void MiniIcon::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
-#ifdef CONFIG_TASKBAR
+void MiniIcon::paint(Graphics &g, const YRect &/*r*/) {
     bool focused = getFrame()->focused();
     YColor *bg = focused ? activeMinimizedWindowBg : normalMinimizedWindowBg;;
     YColor *fg = focused ? activeMinimizedWindowFg : normalMinimizedWindowFg;;
@@ -70,7 +72,7 @@ void MiniIcon::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/,
         g.fillRect(x + 2, y + 2, w - 2, h - 2);
     }
 
-    if (getFrame()->clientIcon() && getFrame()->clientIcon()->small()) {
+    if (getFrame()->clientIcon() && getFrame()->clientIcon()->small() != null) {
         //int y = (height() - 3 - frame()->clientIcon()->small()->height()) / 2;
         g.drawImage(getFrame()->clientIcon()->small(), 2 + tx + 1, 4);
     }
@@ -82,17 +84,16 @@ void MiniIcon::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/,
 
     if (str) {
         g.setColor(fg);
-        YFont *font = minimizedWindowFont;
-        if (font) {
+        ref<YFont> font = minimizedWindowFont;
+        if (font != null) {
             g.setFont(font);
             int ty = (height() - 1 + font->height()) / 2 - font->descent();
             if (ty < 2) ty = 2;
 
-	    g.drawStringEllipsis(tx + 4 + YIcon::sizeSmall + 2, ty,
-				 str, w - 4 - YIcon::sizeSmall - 4);
+	    g.drawStringEllipsis(tx + 4 + YIcon::smallSize() + 2, ty,
+				 str, w - 4 - YIcon::smallSize() - 4);
         }
     }
-#endif
 }
 
 void MiniIcon::handleButton(const XButtonEvent &button) {
@@ -108,7 +109,7 @@ void MiniIcon::handleButton(const XButtonEvent &button) {
     } else if (button.type == ButtonRelease) {
         if (button.button == 1) {
             if (selected == 2) {
-                if (button.state & app->AltMask) {
+                if (button.state & xapp->AltMask) {
                     getFrame()->wmLower();
                 } else {
                     if (!(button.state & ControlMask))
@@ -125,7 +126,7 @@ void MiniIcon::handleButton(const XButtonEvent &button) {
 
 void MiniIcon::handleClick(const XButtonEvent &up, int /*count*/) {
     if (up.button == 3) {
-        getFrame()->popupSystemMenu(up.x_root, up.y_root, -1, -1,
+        getFrame()->popupSystemMenu(this, up.x_root, up.y_root,
                                     YPopupWindow::pfCanFlipVertical |
                                     YPopupWindow::pfCanFlipHorizontal |
                                     YPopupWindow::pfPopupMenu);
@@ -153,9 +154,8 @@ void MiniIcon::handleDrag(const XButtonEvent &down, const XMotionEvent &motion)
         //x += down.x;
         //y += down.y;
 
-        long l = getFrame()->getLayer();
-        int mx = manager->minX(l), Mx = manager->maxX(l);
-        int my = manager->minY(l), My = manager->maxY(l);
+        int mx, my, Mx, My;
+        manager->getWorkArea(getFrame(), &mx, &my, &Mx, &My);
 
         if (x + int(width()) >= Mx) x = Mx - width();
         if (y + int(height()) >= My) y = My - height();
