@@ -9,10 +9,12 @@
 #include "ybutton.h"
 #include "yaction.h"
 #include "ymenu.h"
+#include "yrect.h"
 
-#include "yapp.h" // !!! remove (AltMask)
+#include "yxapp.h" // !!! remove (AltMask)
+#include "yprefs.h"
 #include "prefs.h"
-
+#include "ascii.h"
 #include "wmtaskbar.h"
 
 #include <string.h>
@@ -25,28 +27,29 @@ YColor *YButton::normalButtonFg = 0;
 YColor *YButton::activeButtonBg = 0;
 YColor *YButton::activeButtonFg = 0;
 
-YFont *YButton::normalButtonFont = 0;
-YFont *YButton::activeButtonFont = 0;
+ref<YFont> YButton::normalButtonFont;
+ref<YFont> YButton::activeButtonFont;
 
 // !!! needs to go away
-YPixmap *taskbuttonPixmap = 0;
-YPixmap *taskbuttonactivePixmap = 0;
-YPixmap *taskbuttonminimizedPixmap = 0;
+ref<YPixmap> taskbuttonPixmap;
+ref<YPixmap> taskbuttonactivePixmap;
+ref<YPixmap> taskbuttonminimizedPixmap;
 
 YButton::YButton(YWindow *parent, YAction *action, YMenu *popup) :
     YWindow(parent),
+    fOver(false),
     fAction(action), fPopup(popup),
-    fImage(NULL), fText(NULL),
+    fImage(null), fText(NULL),
     fPressed(false),
     fHotCharPos(-1), hotKey(-1),
     fListener(NULL),
     fSelected(false), fArmed(false),
     wasPopupActive(false),
     fPopupActive(false) {
-    if (normalButtonFont == 0)
-        normalButtonFont = YFont::getFont(normalButtonFontName);
-    if (activeButtonFont == 0)
-        activeButtonFont = YFont::getFont(activeButtonFontName);
+    if (normalButtonFont == null)
+        normalButtonFont = YFont::getFont(XFA(normalButtonFontName));
+    if (activeButtonFont == null)
+        activeButtonFont = YFont::getFont(XFA(activeButtonFontName));
     if (normalButtonBg == 0)
         normalButtonBg = new YColor(clrNormalButton);
     if (normalButtonFg == 0)
@@ -60,27 +63,23 @@ YButton::YButton(YWindow *parent, YAction *action, YMenu *popup) :
 YButton::~YButton() {
     if (hotKey != -1) {
         removeAccelerator(hotKey, 0, this);
-        if (app->AltMask != 0)
-            removeAccelerator(hotKey, app->AltMask, this);
+        if (xapp->AltMask != 0)
+            removeAccelerator(hotKey, xapp->AltMask, this);
     }
     popdown();
     delete fText; fText = 0;
 }
 
-#ifndef CONFIG_TASKBAR
-void YButton::paint(Graphics &, int, int, unsigned, unsigned) {}
-void YButton::paintFocus(Graphics &, int, int, unsigned, unsigned) {}
-#else
-void YButton::paint(Graphics &g, int const d, int const x, int const y,
-				 unsigned const w, unsigned const h) {
+void YButton::paint(Graphics &g, int const d, const YRect &r) {
+    int x = r.x(), y = r.y(), w = r.width(), h = r.height();
     YSurface surface(getSurface());
     g.drawSurface(surface, x, y, w, h);
 
-    if (fImage)
+    if (fImage != null)
         g.drawImage(fImage, x + (w - fImage->width()) / 2,
 			    y + (h - fImage->height()) / 2);
     else if (fText) {
-        YFont *font(fPressed ? activeButtonFont : normalButtonFont);
+        ref<YFont> font = fPressed ? activeButtonFont : normalButtonFont;
 
 	int const w(font->textWidth(fText));
 	int const p((width() - w) / 2);
@@ -95,7 +94,7 @@ void YButton::paint(Graphics &g, int const d, int const x, int const y,
     }
 }
 
-void YButton::paint(Graphics &g, int /*x*/, int /*y*/, unsigned /*w*/, unsigned /*h*/) {
+void YButton::paint(Graphics &g, const YRect &/*r*/) {
     int d((fPressed || fArmed) ? 1 : 0);
     int x(0), y(0), w(width()), h(height());
     
@@ -114,12 +113,12 @@ void YButton::paint(Graphics &g, int /*x*/, int /*y*/, unsigned /*w*/, unsigned
             x += 1 + d; y += 1 + d; w -= 3; h -= 3;
 	}
     
-	paint(g, d, x, y, w, h);
-	paintFocus(g, x, y, w, h);
+	paint(g, d, YRect(x, y, w, h));
+	paintFocus(g, YRect(x, y, w, h));
     }
 }
 
-void YButton::paintFocus(Graphics &g, int /*x*/, int /*y*/, unsigned /*w*/, unsigned /*h*/) {
+void YButton::paintFocus(Graphics &g, const YRect &/*r*/) {
     int const d = (fPressed || fArmed ? 1 : 0);
     int const dp(wmLook == lookMetal ? 2 : 2 + d);
     int const ds(4);
@@ -142,13 +141,19 @@ void YButton::paintFocus(Graphics &g, int /*x*/, int /*y*/, unsigned /*w*/, unsi
         g.setClipRects(0, 0, focus, 4, YXSorted);
 
 	if (wmLook == lookMetal)
-	    paint(g, 0, dp, dp, width() - ds, height() - ds);
+	    paint(g, 0, YRect(dp, dp, width() - ds, height() - ds));
 	else
-	    paint(g, d, dp - 1, dp - 1, width() - ds + 1, height() - ds + 1);
+	    paint(g, d, YRect(dp - 1, dp - 1, width() - ds + 1, height() - ds + 1));
 	g.setClipMask(None);
     }
 }
-#endif
+
+void YButton::setOver(bool over) {
+    if (fOver != over) {
+       fOver= over;
+       repaint();
+    }
+}
 
 void YButton::setPressed(int pressed) {
     if (fPressed != pressed) {
@@ -177,14 +182,14 @@ void YButton::setArmed(bool armed, bool mouseDown) {
 }
 
 bool YButton::handleKey(const XKeyEvent &key) {
-    KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+    KeySym k = XKeycodeToKeysym(xapp->display(), key.keycode, 0);
     unsigned m = KEY_MODMASK(key.state);
-    int uk = TOUPPER(k);
+    int uk = ASCII::toUpper(k);
 
     if (key.type == KeyPress) {
         if (!fSelected) {
             if (((k == XK_Return || k == 32) && m == 0) ||
-                (uk == hotKey && (m & ~app->AltMask) == 0))
+                (uk == hotKey && (m & ~xapp->AltMask) == 0))
             {
                 requestFocus();
                 wasPopupActive = fArmed;
@@ -198,14 +203,14 @@ bool YButton::handleKey(const XKeyEvent &key) {
 
         if (fSelected) {
             if (((k == XK_Return || k == 32) && m == 0) ||
-                (uk == hotKey && (m & ~app->AltMask) == 0))
+                (uk == hotKey && (m & ~xapp->AltMask) == 0))
             {
                 bool wasArmed = fArmed;
 
                 // !!! is this guaranteed to work? (skip autorepeated keys)
                 XEvent xev;
 
-                XCheckTypedWindowEvent(app->display(), handle(), KeyPress, &xev);
+                XCheckTypedWindowEvent(xapp->display(), handle(), KeyPress, &xev);
                 if (xev.type == KeyPress &&
                     xev.xkey.time == key.time &&
                     xev.xkey.keycode == key.keycode &&
@@ -277,29 +282,31 @@ void YButton::handleCrossing(const XCrossingEvent &crossing) {
                 setArmed(false, true);
         }
     }
+
+    if (crossing.type == EnterNotify) {
+        setOver(true);
+    } else if (crossing.type == LeaveNotify) {
+        setOver(false);
+    }
+
     YWindow::handleCrossing(crossing);
 }
 
-void YButton::setImage(YIcon::Image *image) {
+void YButton::setImage(ref<YIconImage> image) {
     fImage = image;
 
-    if (image)
+    if (image != null)
         setSize(image->width() + 3 + 2 - ((wmLook == lookMetal) ? 1 : 0),
                 image->height() + 3 + 2 - ((wmLook == lookMetal) ? 1 : 0));
 }
 
-#ifndef CONFIG_TASKBAR
-void YButton::setText(const char *str, int /*hotChar*/) {
-#else
 void YButton::setText(const char *str, int hotChar) {
-#endif
     if (hotKey != -1) {
         removeAccelerator(hotKey, 0, this);
-        if (app->AltMask != 0)
-            removeAccelerator(hotKey, app->AltMask, this);
+        if (xapp->AltMask != 0)
+            removeAccelerator(hotKey, xapp->AltMask, this);
     }
     fText = newstr(str);
-#ifdef CONFIG_TASKBAR
     /// fix
     if (fText) {
         int w = activeButtonFont->textWidth(fText);
@@ -316,18 +323,17 @@ void YButton::setText(const char *str, int hotChar) {
         }
 
         hotKey = (fHotCharPos != -1) ? fText[fHotCharPos] : -1;
-        hotKey = TOUPPER(hotKey);
+        hotKey = ASCII::toUpper(hotKey);
 
         if (hotKey != -1) {
             installAccelerator(hotKey, 0, this);
-            if (app->AltMask != 0)
-                installAccelerator(hotKey, app->AltMask, this);
+            if (xapp->AltMask != 0)
+                installAccelerator(hotKey, xapp->AltMask, this);
         }
 
         setSize(3 + w + 4 + 2, 3 + h + 4 + 2);
     } else
         hotKey = -1;
-#endif
 }
 
 void YButton::setPopup(YMenu *popup) {
@@ -356,11 +362,14 @@ void YButton::popup(bool mouseDown) {
         mapToGlobal(x, y);
         updatePopup();
         fPopup->setActionListener(getActionListener());
-        if (fPopup->popup(this, 0,
+        
+	int xiscreen = desktop->getScreenForRect(x, y, width(), height());
+        if (fPopup->popup(this, this, 0,
                           x - 2,
                           y + height(),
                           0,
                           height(),
+                          xiscreen,
                           YPopupWindow::pfCanFlipVertical |
                           (mouseDown ? YPopupWindow::pfButtonDown : 0)))
             fPopupActive = true;
@@ -387,7 +396,7 @@ void YButton::actionPerformed(YAction *action, unsigned modifiers) {
         fListener->actionPerformed(action, modifiers);
 }
 
-YFont * YButton::getFont() {
+ref<YFont> YButton::getFont() {
     return (fPressed ? activeButtonFont : normalButtonFont);
 }
 
