@@ -11,6 +11,7 @@
 #include "ycmdline.h"
 
 #include "base.h"
+#include "ascii.h"
 #include "intl.h"
 
 #include <string.h>
@@ -18,14 +19,14 @@
 int YCommandLine::parse() {
     int rc(0);
 
-    for (int n(1), apos(0); !rc && n < argc; ) {
-	char const * const & arg(argv[n]);
+    for (int n = 1, apos = 0; !rc && n < argc; ) {
+	const char * const *arg = argv + n;
 
-	if (*arg == '-') {
-	    char const * value(NULL);
-	    char const option(getArgument(arg, value));
+	if (**arg == '-') {
+	    char const * value = NULL;
+	    char const option = getArgument(*arg, value);
 
-	    rc = setOption(arg, option, value);
+	    rc = setOption(*arg, option, value);
 
 	    if (option != '\0')
 		eatArgument(n);
@@ -52,7 +53,7 @@ char const * YCommandLine::getValue(char const * const & arg,
 				    char const * vptr) {
     if (vptr) { // ------------------------------------- eat leading garbage ---
 	if (*vptr == '=') ++vptr;
-	while (ISBLANK(*vptr)) ++vptr;
+        while (ASCII::isSpaceOrTab(*vptr)) ++vptr;
     } else { // ------------------------- value assumed in the next argument ---
 	int idx(&arg - argv + 1);
 
