@@ -11,14 +11,14 @@
 #include "ymenu.h"
 #include "ymenuitem.h"
 
-#include "yapp.h"
+#include "yxapp.h"
 #include "prefs.h"
 
 #include <string.h>
 
 #include "intl.h"
 
-YFont *YInputLine::inputFont = 0;
+ref<YFont> YInputLine::inputFont;
 YColor *YInputLine::inputBg = 0;
 YColor *YInputLine::inputFg = 0;
 YColor *YInputLine::inputSelectionBg = 0;
@@ -31,8 +31,8 @@ int YInputLine::fAutoScrollDelta = 0;
 static YAction *actionCut, *actionCopy, *actionPaste, *actionSelectAll, *actionPasteSelection;
 
 YInputLine::YInputLine(YWindow *parent): YWindow(parent) {
-    if (inputFont == 0)
-        inputFont = YFont::getFont(inputFontName);
+    if (inputFont == null)
+        inputFont = YFont::getFont(XFA(inputFontName));
     if (inputBg == 0)
         inputBg = new YColor(clrInput);
     if (inputFg == 0)
@@ -66,7 +66,7 @@ YInputLine::YInputLine(YWindow *parent): YWindow(parent) {
     fHasFocus = false;
     fSelecting = false;
     fCursorVisible = true;
-    if (inputFont)
+    if (inputFont != null)
         setSize(width(), inputFont->height() + 2);
 }
 YInputLine::~YInputLine() {
@@ -93,8 +93,8 @@ const char *YInputLine::getText() {
     return fText;
 }
 
-void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
-    YFont *font = inputFont;
+void YInputLine::paint(Graphics &g, const YRect &/*r*/) {
+    ref<YFont> font = inputFont;
     int min, max, minOfs = 0, maxOfs = 0;
     int textLen = fText ? strlen(fText) : 0;
 
@@ -106,7 +106,7 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
         max = markPos;
     }
 
-    if (curPos == markPos || !fText || !font || !fHasFocus) {
+    if (curPos == markPos || !fText || font == null || !fHasFocus) {
         g.setColor(inputBg);
         g.fillRect(0, 0, width(), height());
     } else {
@@ -128,7 +128,7 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
         }
     }
 
-    if (font) {
+    if (font != null) {
         int yp = 1 + font->ascent();
         int curOfs = fText ? font->textWidth(fText, curPos) : 0;
         int cx = curOfs - leftOfs;
@@ -161,7 +161,10 @@ void YInputLine::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/
 
 bool YInputLine::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        int index = 0;
+        if (key.state & xapp->NumLockMask)
+            index = 1;
+        KeySym k = XKeycodeToKeysym(xapp->display(), key.keycode, index);
         int m = KEY_MODMASK(key.state);
         bool extend = (m & ShiftMask) ? true : false;
         int textLen = fText ? strlen(fText) : 0;
@@ -377,7 +380,7 @@ void YInputLine::handleClickDown(const XButtonEvent &down, int count) {
 void YInputLine::handleClick(const XButtonEvent &up, int /*count*/) {
     if (up.button == 3 && IS_BUTTON(up.state, Button3Mask)) {
         if (inputMenu)
-            inputMenu->popup(0, 0, up.x_root, up.y_root, -1, -1,
+            inputMenu->popup(this, 0, 0, up.x_root, up.y_root,
                              YPopupWindow::pfCanFlipVertical |
                              YPopupWindow::pfCanFlipHorizontal);
     } else if (up.button == 2 && IS_BUTTON(up.state, Button2Mask)) {
@@ -393,7 +396,7 @@ void YInputLine::handleSelection(const XSelectionEvent &selection) {
         long nitems;
         char *data;
 
-        XGetWindowProperty(app->display(),
+        XGetWindowProperty(xapp->display(),
                            selection.requestor, selection.property,
                            0L, 32 * 1024, True,
                            selection.target, &type, &format,
@@ -410,11 +413,11 @@ void YInputLine::handleSelection(const XSelectionEvent &selection) {
 }
 
 int YInputLine::offsetToPos(int offset) {
-    YFont *font = inputFont;
+    ref<YFont> font = inputFont;
     int ofs = 0, pos = 0;;
     int textLen = fText ? strlen(fText) : 0;
 
-    if (font) {
+    if (font != null) {
         while (pos < textLen) {
             ofs += font->textWidth(fText + pos, 1);
             if (ofs < offset)
@@ -504,17 +507,17 @@ void YInputLine::limit() {
     if (markPos < 0)
         markPos = 0;
 
-    YFont *font = inputFont;
-    if (font) {
+    ref<YFont> font = inputFont;
+    if (font != null) {
         int curOfs = font->textWidth(fText, curPos);
         int curLen = font->textWidth(fText, textLen);
 
-        if (curOfs >= leftOfs + int(width()))
-            leftOfs = curOfs - width() + 1;
+        if (curOfs >= leftOfs + int(width()) + 1)
+            leftOfs = curOfs - width() + 2;
         if (curOfs < leftOfs)
             leftOfs = curOfs;
-        if (leftOfs + int(width()) > curLen)
-            leftOfs = curLen - width();
+        if (leftOfs + int(width()) + 1 > curLen)
+            leftOfs = curLen - width() + 1;
         if (leftOfs < 0)
             leftOfs = 0;
     }
@@ -679,7 +682,7 @@ void YInputLine::copySelection() {
             min = curPos;
             max = markPos;
         }
-        app->setClipboardText(fText + min, max - min);
+        xapp->setClipboardText(fText + min, max - min);
     }
 }
 
