@@ -12,15 +12,18 @@
 #include "ypixbuf.h"
 #include "ykey.h"
 #include "ylistbox.h"
+#include "yrect.h"
+#include "yicon.h"
 
 #include "yscrollview.h"
 
-#include "yapp.h"
+#include "yxapp.h"
 #include "prefs.h"
+#include "ascii.h"
 
 #include <string.h>
 
-static YFont *listBoxFont = 0;
+static ref<YFont> listBoxFont;
 static YColor *listBoxBg = 0;
 static YColor *listBoxFg = 0;
 static YColor *listBoxSelBg = 0;
@@ -70,8 +73,8 @@ int YListItem::getOffset() {
 
 YListBox::YListBox(YScrollView *view, YWindow *aParent): 
     YWindow(aParent) INIT_GRADIENT(fGradient, NULL) {
-    if (listBoxFont == 0)
-        listBoxFont = YFont::getFont(listBoxFontName);
+    if (listBoxFont == null)
+        listBoxFont = YFont::getFont(XFA(listBoxFontName));
     if (listBoxBg == 0)
         listBoxBg = new YColor(clrListBox);
     if (listBoxFg == 0)
@@ -108,7 +111,7 @@ YListBox::~YListBox() {
     fFirst = fLast = 0;
     freeItems();
 #ifdef CONFIG_GRADIENTS
-    delete fGradient;
+    fGradient = null;
 #endif
 }
 
@@ -188,7 +191,7 @@ void YListBox::updateItems() {
                 fItems[n++] = a;
 
                 int cw = 3 + 20 + a->getOffset();
-                if (listBoxFont) {
+                if (listBoxFont != null) {
                     const char *t = a->getText();
                     if (t)
                         cw += listBoxFont->textWidth(t) + 3;
@@ -246,7 +249,7 @@ YListItem *YListBox::getItem(int no) {
 }
 
 int YListBox::getLineHeight() {
-    return max((int) YIcon::sizeSmall, (int) listBoxFont->height()) + 2;
+    return max((int) YIcon::smallSize(), (int) listBoxFont->height()) + 2;
 }
 
 void YListBox::ensureVisibility(int item) { //!!! horiz too
@@ -280,21 +283,20 @@ void YListBox::focusVisible() {
     }
 }
 
-void YListBox::configure(const int x, const int y, 
-			 const unsigned width, const unsigned height, 
-			 const bool resized) {
-    YWindow::configure(x, y, width, height, resized);
+void YListBox::configure(const YRect &r, const bool resized) {
+    YWindow::configure(r, resized);
     //if (fFocusedItem != -1)
     //    paintItem(fFocusedItem);
     if (resized) {
         resetScrollBars();
 
 #ifdef CONFIG_GRADIENTS
-	if (listbackPixbuf && !(fGradient &&
-				 fGradient->width() == width &&
-				 fGradient->height() == height)) {
-	    delete fGradient;
-	    fGradient = new YPixbuf(*listbackPixbuf, width, height);
+        if (listbackPixbuf != null
+            && !(fGradient != null &&
+                 fGradient->width() == r.width() &&
+                 fGradient->height() == r.height()))
+        {
+            fGradient = YPixbuf::scale(listbackPixbuf, r.width(), r.height());
 	    repaint();
 	}
 #endif
@@ -303,7 +305,7 @@ void YListBox::configure(const int x, const int y,
 
 bool YListBox::handleKey(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        KeySym k = XKeycodeToKeysym(xapp->display(), key.keycode, 0);
         int m = KEY_MODMASK(key.state);
 
         bool clear = (m & ControlMask) ? false : true;
@@ -393,7 +395,7 @@ bool YListBox::handleKey(const XKeyEvent &key) {
             }
         default:
             if (k < 256) {
-                unsigned char c = TOUPPER(k);
+                unsigned char c = ASCII::toUpper(k);
                 int count = getItemCount();
                 int i = fFocusedItem;
                 YListItem *it = 0;
@@ -403,7 +405,7 @@ bool YListBox::handleKey(const XKeyEvent &key) {
                     i = (i + 1) % count;
                     it = getItem(i);
                     title = it->getText();
-                    if (title && TOUPPER(title[0]) == c) {
+                    if (title && ASCII::toUpper(title[0]) == c) {
                         setFocusedItem(i, clear, extend, false);
                         break;
                     }
@@ -573,12 +575,12 @@ void YListBox::paintItem(Graphics &g, int n) {
         g.fillRect(0, y - fOffsetY, width(), lh);
     } else {
 #ifdef CONFIG_GRADIENTS
-        if (fGradient)
+        if (fGradient != null)
 	    g.copyPixbuf(*fGradient, 0, y - fOffsetY, width(), lh,
 	    			     0, y - fOffsetY);
         else 
 #endif	
-	if (listbackPixmap)
+	if (listbackPixmap != null)
 	    g.fillPixmap(listbackPixmap, 0, y - fOffsetY, width(), lh);
 	else {
 	    g.setColor(listBoxBg);
@@ -590,7 +592,7 @@ void YListBox::paintItem(Graphics &g, int n) {
         g.setColor(YColor::black);
         g.setPenStyle(true);
         int cw = 3 + 20 + a->getOffset();
-        if (listBoxFont) {
+        if (listBoxFont != null) {
             const char *t = a->getText();
             if (t)
                 cw += listBoxFont->textWidth(t) + 3;
@@ -601,7 +603,7 @@ void YListBox::paintItem(Graphics &g, int n) {
 
     YIcon *icon = a->getIcon();
 
-    if (icon && icon->small())
+    if (icon && icon->small() != null)
         g.drawImage(icon->small(), xpos + x - fOffsetX, y - fOffsetY + 1);
 
     const char *title = a->getText();
@@ -615,7 +617,8 @@ void YListBox::paintItem(Graphics &g, int n) {
     }
 }
 
-void YListBox::paint(Graphics &g, int /*x*/, int ry, unsigned int /*width*/, unsigned int rheight) {
+void YListBox::paint(Graphics &g, const YRect &r) {
+    int ry = r.y(), rheight = r.height();
     int const lh(getLineHeight());
     int const min((fOffsetY + ry) / lh);
     int const max((fOffsetY + ry + rheight) / lh);
@@ -623,15 +626,15 @@ void YListBox::paint(Graphics &g, int /*x*/, int ry, unsigned int /*width*/, uns
     for (int n(min); n <= max; n++) paintItem(g, n);
     resetScrollBars();
 
-    unsigned const y(contentHeight());
+    int const y(contentHeight());
 
     if (y < height()) {
 #ifdef CONFIG_GRADIENTS
-        if (fGradient)
+        if (fGradient != null)
             g.copyPixbuf(*fGradient, 0, y, width(), height() - y, 0, y);
         else 
 #endif	
-	if (listbackPixmap)
+	if (listbackPixmap != null)
             g.fillPixmap(listbackPixmap, 0, y, width(), height() - y);
         else {
 	    g.setColor(listBoxBg);
@@ -641,6 +644,7 @@ void YListBox::paint(Graphics &g, int /*x*/, int ry, unsigned int /*width*/, uns
 }
 
 void YListBox::paintItem(int i) {
+#warning "fix this to use an invalidate region"
     if (i >= 0 && i < getItemCount())
         paintItem(getGraphics(), i);
 }
