@@ -30,12 +30,14 @@
  */
 
 #include "config.h"
-#include "default.h"
-#include "intl.h"
 
 #include "ypixbuf.h"
 #include "base.h"
-#include "yapp.h"
+#include "yxapp.h"
+
+#include "yprefs.h"
+#include "default.h"
+#include "intl.h"
 
 #include "X11/X.h"
 
@@ -67,9 +69,9 @@ bool YPixbuf::init() {
         parms.imagecachesize = 0;
         parms.pixmapcachesize = 0;
 
-        hImlib = Imlib_init_with_params(app->display(), &parms);
+        hImlib = Imlib_init_with_params(xapp->display(), &parms);
     } else
-        hImlib = Imlib_init(app->display());
+        hImlib = Imlib_init(xapp->display());
 
     return hImlib;
 }
@@ -84,12 +86,13 @@ bool YPixbuf::init() {
  * A scaler for Grayscale/RGB/RGBA pixel buffers
  ******************************************************************************/
 
+typedef unsigned long fixed;
+static fixed const fUnit = 1L << 12;
+static fixed const fPrec = 12;
+enum { R, G, B, A };
+
 template <class Pixel, int Channels> class YScaler {
 public:
-    typedef unsigned long fixed;
-    static fixed const fUnit = 1L << 12;
-    static fixed const fPrec = 12;
-    enum { R, G, B, A };
 
     YScaler(Pixel const * src, unsigned const sStep,
             unsigned const sw, unsigned const sh,
@@ -368,6 +371,8 @@ YScaler<Pixel, Channels>::YScaler
  * A scaler for RGB pixel buffers
  ******************************************************************************/
 
+#warning "fix the optimized versions"
+#if 0
 template <int Channels>
 static void copyRGB32ToPixbuf(char const * src, unsigned const sStep,
                               unsigned char * dst, unsigned const dStep,
@@ -432,33 +437,41 @@ static void copyRGB444ToPixbuf(char const * src, unsigned const sStep,
         }
     }
 }
+#endif
 
 template <class Pixel, int Channels>
-static void copyRGBAnyToPixbuf(char const * src, unsigned const sStep,
+static void copyRGBAnyToPixbuf(XImage *image, char const * src, unsigned const sStep,
                                unsigned char * dst, unsigned const dStep,
                                unsigned const width, unsigned const height,
                                unsigned const rMask, unsigned const gMask,
-                               unsigned const bMask) {
-    warn(_("Using fallback mechanism to convert pixels "
+                               unsigned const bMask)
+{
+    MSG((_("Using fallback mechanism to convert pixels "
            "(depth: %d; masks (red/green/blue): %0*x/%0*x/%0*x)"),
-           sizeof(Pixel) * 8,
-           sizeof(Pixel) * 2, rMask, sizeof(Pixel) * 2, gMask,
-           sizeof(Pixel) * 2, bMask);
+         sizeof(Pixel) * 8,
+         sizeof(Pixel) * 2, rMask, sizeof(Pixel) * 2, gMask,
+         sizeof(Pixel) * 2, bMask));
 
-    unsigned const rShift(lowbit(rMask));
-    unsigned const gShift(lowbit(gMask));
-    unsigned const bShift(lowbit(bMask));
+    unsigned const rShift = lowbit(rMask);
+    unsigned const gShift = lowbit(gMask);
+    unsigned const bShift = lowbit(bMask);
 
-    unsigned const rLoss(7 + rShift - highbit(rMask));
-    unsigned const gLoss(7 + gShift - highbit(gMask));
-    unsigned const bLoss(7 + bShift - highbit(bMask));
+    unsigned const rLoss = 7 + rShift - highbit(rMask);
+    unsigned const gLoss = 7 + gShift - highbit(gMask);
+    unsigned const bLoss = 7 + bShift - highbit(bMask);
 
-    for (unsigned y(height); y > 0; --y, src+= sStep, dst+= dStep) {
-        Pixel const * s((Pixel*)src); unsigned char * d(dst);
-        for (unsigned x(width); x-- > 0; d+= Channels, ++s) {
-            d[0] = ((*s & rMask) >> rShift) << rLoss;
-            d[1] = ((*s & gMask) >> gShift) << gLoss;
-            d[2] = ((*s & bMask) >> bShift) << bLoss;
+    unsigned char *d = dst;
+    for (unsigned y = height; y > 0; --y, src += sStep, dst += dStep) {
+        for (unsigned x = width; x > 0; x--, d += Channels) {
+            unsigned long pixel = XGetPixel(image, width - x, height - y);
+
+            ///Pixel const * s = (Pixel*)src;
+
+            d[0] = ((pixel & rMask) >> rShift) << rLoss;
+            d[1] = ((pixel & gMask) >> gShift) << gLoss;
+            d[2] = ((pixel & bMask) >> bShift) << bLoss;
+            if (Channels == 4)
+                d[3] = 0;
         }
     }
 }
@@ -488,23 +501,26 @@ static YPixbuf::Pixel * copyImageToPixbuf(XImage & image,
     YPixbuf::Pixel * pixels = new YPixbuf::Pixel[rowstride * height];
 
     if (!(image.red_mask && image.green_mask && image.blue_mask)) {
-        Visual const * visual(app->visual());
+        Visual const * visual(xapp->visual());
         image.red_mask = visual->red_mask;
         image.green_mask = visual->green_mask;
         image.blue_mask = visual->blue_mask;
     }
 
     if (image.depth > 16) {
+#if 0
         if (CHANNEL_MASK(image, 0xff0000, 0x00ff00, 0x0000ff) ||
             CHANNEL_MASK(image, 0x0000ff, 0x00ff00, 0xff0000))
             copyRGB32ToPixbuf<Channels> (image.data, image.bytes_per_line,
                                          pixels, rowstride, width, height);
         else
+#endif
             copyRGBAnyToPixbuf<yuint32, Channels>
-                (image.data, image.bytes_per_line,
+                (&image, image.data, image.bytes_per_line,
                  pixels, rowstride, width, height,
                  image.red_mask, image.green_mask, image.blue_mask);
     } else if (image.depth > 8) {
+#if 0
         if (CHANNEL_MASK(image, 0xf800, 0x07e0, 0x001f) ||
             CHANNEL_MASK(image, 0x001f, 0x07e0, 0xf800))
             copyRGB565ToPixbuf<Channels> (image.data, image.bytes_per_line,
@@ -518,8 +534,9 @@ static YPixbuf::Pixel * copyImageToPixbuf(XImage & image,
             copyRGB444ToPixbuf<Channels> (image.data, image.bytes_per_line,
                                           pixels, rowstride, width, height);
         else
+#endif
             copyRGBAnyToPixbuf<yuint16, Channels>
-                (image.data, image.bytes_per_line,
+                (&image, image.data, image.bytes_per_line,
                  pixels, rowstride, width, height,
                  image.red_mask, image.green_mask, image.blue_mask);
     } else
@@ -670,7 +687,7 @@ static void copyPixbufToImage(YPixbuf::Pixel const * pixels,
  ******************************************************************************/
 
 void YPixbuf::copyArea(YPixbuf const & src, int sx, int sy,
-                       unsigned w, unsigned h, int dx, int dy) {
+                       int w, int h, int dx, int dy) {
     if (sx < 0) { dx-= sx; w+= sx; sx = 0; }
     if (sy < 0) { dy-= sy; h+= sy; sy = 0; }
     if (dx < 0) { sx-= dx; w+= dx; dx = 0; }
@@ -734,7 +751,7 @@ void YPixbuf::copyArea(YPixbuf const & src, int sx, int sy,
 }
 
 void YPixbuf::copyAlphaToMask(Pixmap pixmap, GC gc, int sx, int sy,
-                              unsigned w, unsigned h, int dx, int dy) {
+                              int w, int h, int dx, int dy) {
     if (sx < 0) { dx-= sx; w+= sx; sx = 0; }
     if (sy < 0) { dy-= sy; h+= sy; sy = 0; }
     if (dx < 0) { sx-= dx; w+= dx; dx = 0; }
@@ -743,11 +760,11 @@ void YPixbuf::copyAlphaToMask(Pixmap pixmap, GC gc, int sx, int sy,
     w = min(w, width());
     h = min(h, height());
 
-    XSetForeground(app->display(), gc, 1);
-    XFillRectangle(app->display(), pixmap, gc, dx, dy, w, h);
+    XSetForeground(xapp->display(), gc, 1);
+    XFillRectangle(xapp->display(), pixmap, gc, dx, dy, w, h);
 
     if (alpha()) {
-        XSetForeground(app->display(), gc, 0);
+        XSetForeground(xapp->display(), gc, 0);
 
         unsigned const delta(inlineAlpha() ? 4 : 3);
         unsigned const rowStride(inlineAlpha() ? rowstride() : width());
@@ -756,9 +773,9 @@ void YPixbuf::copyAlphaToMask(Pixmap pixmap, GC gc, int sx, int sy,
         for (unsigned y(h); y; --y, row+= rowStride, ++dy) {
             Pixel const * pixel(row);
 
-            for (unsigned xa(0), xe(0); xe < w; xa = xe + 1) {
+            for (int xa(0), xe(0); xe < w; xa = xe + 1) {
                 while (xe < w && *pixel++ < 128) ++xe;
-                XFillRectangle(app->display(), pixmap, gc,
+                XFillRectangle(xapp->display(), pixmap, gc,
                                dx + xa, dy, xe - xa, 1);
                 while (xe < w && *pixel++ >= 128) ++xe;
             }
@@ -776,17 +793,18 @@ void YPixbuf::copyAlphaToMask(Pixmap pixmap, GC gc, int sx, int sy,
 
 /******************************************************************************/
 
-YPixbuf::YPixbuf(char const * filename, bool fullAlpha):
+YPixbuf::YPixbuf(char const *filename, bool fullAlpha):
     fWidth(0), fHeight(0), fRowStride(0),
-    fPixels(NULL), fAlpha(NULL), fPixmap(None) {
+    fPixels(NULL), fAlpha(NULL), fPixmap(None)
+{
     XpmAttributes xpmAttributes;
     memset(&xpmAttributes, 0, sizeof(xpmAttributes));
-    xpmAttributes.colormap  = app->colormap();
+    xpmAttributes.colormap  = xapp->colormap();
     xpmAttributes.closeness = 65535;
     xpmAttributes.valuemask = XpmSize|XpmReturnPixels|XpmColormap|XpmCloseness;
 
     XImage * image(NULL), * alpha(NULL);
-    int const rc(XpmReadFileToImage(app->display(),
+    int const rc(XpmReadFileToImage(xapp->display(),
                                     (char *)REDIR_ROOT(filename), // !!!
                                     &image, &alpha, &xpmAttributes));
 
@@ -813,43 +831,71 @@ YPixbuf::YPixbuf(char const * filename, bool fullAlpha):
     if (alpha) XDestroyImage(alpha);
 }
 
-YPixbuf::YPixbuf(unsigned const width, unsigned const height):
+#if 0
+YPixbuf::YPixbuf(char const *filename, int w, int h, bool fullAlpha):
+    fWidth(w), fHeight(h), fRowStride(0),
+    fPixels(NULL), fAlpha(NULL), fPixmap(None)
+{
+    ref<YPixbuf> source;
+    source.init(new YPixbuf(filename, fullAlpha));
+    if (source != null && source->valid()) {
+        fRowStride = ((w * (source->alpha() ? 4 : 3) + 3) & ~3),
+        fPixels = new Pixel[fRowStride * fHeight];
+
+        if (source->alpha()) {
+            fAlpha = fPixels + 3;
+            YScaler<Pixel, 4>(source->pixels(), source->rowstride(),
+                              source->width(), source->height(),
+                              fPixels, fRowStride, fWidth, fHeight);
+        } else {
+            YScaler<Pixel, 3>(source->pixels(), source->rowstride(),
+                              source->width(), source->height(),
+                              fPixels, fRowStride, fWidth, fHeight);
+        }
+    }
+}
+#endif
+
+YPixbuf::YPixbuf(int const width, int const height):
     fWidth(width), fHeight(height), fRowStride((width * 3 + 3) & ~3),
     fPixels(NULL), fAlpha(NULL), fPixmap(None) {
     fPixels = new Pixel[fRowStride * fHeight];
 }
 
-YPixbuf::YPixbuf(YPixbuf const & source,
-                 unsigned const width, unsigned const height):
+YPixbuf::YPixbuf(const ref<YPixbuf> &source,
+                 int const width, int const height):
     fWidth(width), fHeight(height),
-    fRowStride((width * (source.alpha() ? 4 : 3) + 3) & ~3),
-    fPixels(NULL), fAlpha(NULL), fPixmap(None) {
-
-    if (source) {
+    fRowStride((width * (source->alpha() ? 4 : 3) + 3) & ~3),
+    fPixels(NULL), fAlpha(NULL), fPixmap(None)
+{
+    if (source != null && source->valid()) {
         fPixels = new Pixel[fRowStride * fHeight];
 
-        if (source.alpha()) {
+        if (source->alpha()) {
             fAlpha = fPixels + 3;
-            YScaler<Pixel, 4>(source.pixels(), source.rowstride(),
-                              source.width(), source.height(),
+            YScaler<Pixel, 4>(source->pixels(), source->rowstride(),
+                              source->width(), source->height(),
                               fPixels, fRowStride, fWidth, fHeight);
         } else
-            YScaler<Pixel, 3>(source.pixels(), source.rowstride(),
-                              source.width(), source.height(),
+            YScaler<Pixel, 3>(source->pixels(), source->rowstride(),
+                              source->width(), source->height(),
                               fPixels, fRowStride, fWidth, fHeight);
     }
 }
 
 YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
-                 unsigned w, unsigned h, int x, int y,
-                 bool fullAlpha) :
+                 int dWidth, int dHeight, int w, int h, int x, int y,
+                 bool fullAlpha):
     fWidth(0), fHeight(0), fRowStride(0),
-    fPixels(NULL), fAlpha(NULL), fPixmap(None) {
+    fPixels(NULL), fAlpha(NULL), fPixmap(None)
+{
 
-    Window dRoot; unsigned dWidth, dHeight, dDummy;
-    XGetGeometry(app->display(), drawable, &dRoot,
-                 (int*)&dDummy, (int*)&dDummy,
-                 &dWidth, &dHeight, &dDummy, &dDummy);
+//#warning "!!! remove call to XGetGeometry"
+//    Window dRoot; int dWidth, dHeight, dDummy;
+//    XGetGeometry(xapp->display(), drawable, &dRoot,
+//                 (int*)&dDummy, (int*)&dDummy,
+//                 (unsigned int*)&dWidth, (unsigned int*)&dHeight,
+//                 (unsigned int*)&dDummy, (unsigned int*)&dDummy);
 
     MSG(("YPixbuf::YPixbuf: initial: x=%i, y=%i; w=%i, h=%i", x, y, w, h));
 
@@ -861,10 +907,10 @@ YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
     MSG(("YPixbuf::YPixbuf: after clipping: x=%i, y=%i; w=%i, h=%i", x, y, w, h));
     if (!(w && h)) return;
 
-    XImage * image(XGetImage(app->display(), drawable, x, y, w, h,
+    XImage * image(XGetImage(xapp->display(), drawable, x, y, w, h,
                              AllPlanes, ZPixmap));
     XImage * alpha(fullAlpha && mask != None ?
-        XGetImage(app->display(), mask, x, y, w, h, AllPlanes, ZPixmap) : NULL);
+        XGetImage(xapp->display(), mask, x, y, w, h, AllPlanes, ZPixmap) : NULL);
 
     if (image) {
         MSG(("depth/padding: %d/%d; r/g/b mask: %d/%d/%d",
@@ -900,30 +946,33 @@ YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
 }
 
 YPixbuf::~YPixbuf() {
-    if (fPixmap != None)
-        XFreePixmap(app->display(), fPixmap);
+    if (fPixmap != None) {
+        if (xapp != 0)
+            XFreePixmap(xapp->display(), fPixmap);
+        fPixmap = 0;
+    }
 
     delete[] fPixels;
 }
 
 void YPixbuf::copyToDrawable(Drawable drawable, GC gc,
                              int const sx, int const sy,
-                             unsigned const w, unsigned const h,
+                             int const w, int const h,
                              int const dx, int const dy, bool useAlpha) {
     if (fPixmap == None && fPixels) {
-        unsigned const depth(app->depth());
+        unsigned const depth(xapp->depth());
         unsigned const pixelSize(depth > 16 ? 4 : depth > 8 ? 2 : 1);
         unsigned const rowStride(((fWidth * pixelSize) + 3) & ~3);
         char * pixels(new char[rowStride * fHeight]);
 
         fPixmap = YPixmap::createPixmap(fWidth, fHeight);
 
-        XImage * image(XCreateImage(app->display(), app->visual(),
+        XImage * image(XCreateImage(xapp->display(), xapp->visual(),
             depth, ZPixmap, 0, pixels, fWidth, fHeight, 32, rowStride));
 
         if (image) {
             copyPixbufToImage<3>(fPixels, *image, fRowStride);
-            Graphics(fPixmap).copyImage(image, 0, 0);
+            Graphics(fPixmap, fWidth, fHeight).copyImage(image, 0, 0);
 
             delete[] image->data;
             image->data = NULL;
@@ -940,7 +989,7 @@ void YPixbuf::copyToDrawable(Drawable drawable, GC gc,
         warn("YPixbuf::copyToDrawable with alpha not implemented yet");
 
     if (fPixmap != None)
-        XCopyArea(app->display(), fPixmap, drawable, gc, sx, sy, w, h, dx, dy);
+        XCopyArea(xapp->display(), fPixmap, drawable, gc, sx, sy, w, h, dx, dy);
 }
 
 #endif
@@ -951,38 +1000,73 @@ void YPixbuf::copyToDrawable(Drawable drawable, GC gc,
 
 #ifdef CONFIG_IMLIB
 
-YPixbuf::YPixbuf(char const * filename, bool fullAlpha):
-    fImage(Imlib_load_image(hImlib, (char*) filename)), fAlpha(NULL) {
+YPixbuf::YPixbuf(char const *filename, bool fullAlpha):
+    fImage(NULL), fAlpha(NULL)
+{
+    fImage = Imlib_load_image(hImlib, (char *)filename);
 
     if (NULL == fImage)
         warn(_("Loading of image \"%s\" failed"), filename);
 
-    if (fullAlpha) allocAlphaChannel();
+    if (fullAlpha)
+        allocAlphaChannel();
+    msg("%s %d %d", filename, width(), height());
 }
 
-YPixbuf::YPixbuf(unsigned const width, unsigned const height):
+#if 0
+YPixbuf::YPixbuf(char const *filename, int w, int h, bool fullAlpha):
+    fImage(NULL), fAlpha(NULL)
+{
+    ref<YPixbuf> source;
+    source.init(new YPixbuf(filename, fullAlpha));
+
+    if (source != null && source->valid()) {
+        if (source->alpha()) {
+            fAlpha = new Pixel[w * h];
+            YScaler<Pixel, 1>(source->alpha(), source->width(),
+                              source->width(), source->height(),
+                              fAlpha, w, w, h);
+        }
+
+        const unsigned rowstride(3 * w);
+        Pixel * pixels(new Pixel[rowstride * h]);
+
+        YScaler<Pixel, 3>(source->pixels(), source->rowstride(),
+                          source->width(), source->height(),
+                          pixels, rowstride, w, h);
+
+        fImage = Imlib_create_image_from_data(hImlib, pixels, NULL,
+                                              w, h);
+
+        delete[] pixels;
+    }
+}
+#endif
+
+YPixbuf::YPixbuf(int const width, int const height):
     fAlpha(NULL) {
     Pixel * empty(new Pixel[width * height * 3]);
     fImage = Imlib_create_image_from_data(hImlib, empty, NULL, width, height);
     delete[] empty;
 }
 
-YPixbuf::YPixbuf(YPixbuf const & source,
-                 unsigned const width, unsigned const height):
-    fImage(NULL), fAlpha(NULL) {
-    if (source) {
-        if (source.alpha()) {
+YPixbuf::YPixbuf(const ref<YPixbuf> &source,
+                 int const width, int const height):
+    fImage(NULL), fAlpha(NULL)
+{
+    if (source != null && source->valid()) {
+        if (source->alpha()) {
             fAlpha = new Pixel[width * height];
-            YScaler<Pixel, 1>(source.alpha(), source.width(),
-                              source.width(), source.height(),
+            YScaler<Pixel, 1>(source->alpha(), source->width(),
+                              source->width(), source->height(),
                               fAlpha, width, width, height);
         }
 
         const unsigned rowstride(3 * width);
         Pixel * pixels(new Pixel[rowstride * height]);
 
-        YScaler<Pixel, 3>(source.pixels(), source.rowstride(),
-                          source.width(), source.height(),
+        YScaler<Pixel, 3>(source->pixels(), source->rowstride(),
+                          source->width(), source->height(),
                           pixels, rowstride, width, height);
 
         fImage = Imlib_create_image_from_data(hImlib, pixels, NULL,
@@ -993,13 +1077,15 @@ YPixbuf::YPixbuf(YPixbuf const & source,
 }
 
 YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
-                 unsigned w, unsigned h, int x, int y,
+                 int dWidth, int dHeight, int w, int h, int x, int y,
                  bool fullAlpha) :
-    fImage(NULL), fAlpha(NULL) {
-    Window dRoot; unsigned dWidth, dHeight, dDummy;
-    XGetGeometry(app->display(), drawable, &dRoot,
-                 (int*)&dDummy, (int*)&dDummy,
-                 &dWidth, &dHeight, &dDummy, &dDummy);
+fImage(NULL), fAlpha(NULL)
+{
+//#warning "!!! remove call to XGetGeometry"
+//    Window dRoot; int dWidth, dHeight, dDummy;
+//    XGetGeometry(xapp->display(), drawable, &dRoot,
+//                 &dDummy, &dDummy,
+//                 (unsigned int*)&dWidth, (unsigned int*)&dHeight, (unsigned int*)&dDummy, (unsigned int*)&dDummy);
 
     MSG(("YPixbuf::YPixbuf: initial: x=%i, y=%i; w=%i, h=%i", x, y, w, h));
 
@@ -1011,7 +1097,7 @@ YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
     MSG(("YPixbuf::YPixbuf: after clipping: x=%i, y=%i; w=%i, h=%i", x, y, w, h));
     if (!(w && h)) return;
 
-    XImage * image(XGetImage(app->display(), drawable, x, y, w, h,
+    XImage * image(XGetImage(xapp->display(), drawable, x, y, w, h,
                              AllPlanes, ZPixmap));
 
     if (image) {
@@ -1023,20 +1109,22 @@ YPixbuf::YPixbuf(Drawable drawable, Pixmap mask,
         fImage = Imlib_create_image_from_data(hImlib, pixels, NULL, w, h);
         delete[] pixels;
         XDestroyImage(image);
-    } else
-        warn(_("%s:%d: Failed to copy drawable 0x%x to pixel buffer"),
-               __FILE__, __LINE__, drawable);
+    } else {
+        warn(_("%s:%d: Failed to copy drawable 0x%x to pixel buffer (%d:%d-%dx%d"),
+             __FILE__, __LINE__, drawable, x, y, w, h);
+    }
 
     if (fullAlpha && mask != None) {
-        image = XGetImage(app->display(), mask, x, y, w, h, AllPlanes, ZPixmap);
+        image = XGetImage(xapp->display(), mask, x, y, w, h, AllPlanes, ZPixmap);
         if (image) {
             fAlpha = new Pixel[w * h];
             copyBitmapToPixbuf<1>(image->data, image->bytes_per_line,
                                   fAlpha, w, w, h);
             XDestroyImage(image);
-        } else
+        } else {
             warn(_("%s:%d: Failed to copy drawable 0x%x to pixel buffer"),
-                   __FILE__, __LINE__, mask);
+                 __FILE__, __LINE__, mask);
+        }
     }
 }
 
@@ -1079,7 +1167,7 @@ void YPixbuf::allocAlphaChannel() {
 
 void YPixbuf::copyToDrawable(Drawable drawable, GC gc,
                              int const sx, int const sy,
-                             unsigned const w, unsigned const h,
+                             int const w, int const h,
                              int const dx, int const dy, bool useAlpha) {
     if (fImage) {
         if (useAlpha && alpha())
@@ -1088,11 +1176,20 @@ void YPixbuf::copyToDrawable(Drawable drawable, GC gc,
         if (fImage->pixmap == None)
             Imlib_render(hImlib, fImage, width(), height());
 
-        XCopyArea(app->display(), fImage->pixmap, drawable, gc,
+        XCopyArea(xapp->display(), fImage->pixmap, drawable, gc,
                                   sx, sy, w, h, dx, dy);
     }
 }
 
 #endif
 
+ref<YPixbuf> YPixbuf::scale(ref<YPixbuf> source, int const w, int const h) {
+    ref<YPixbuf> scaled;
+    if (source->width() != w || source->height() != h) {
+        scaled.init(new YPixbuf(source, w, h));
+    } else
+        scaled = source;
+    return scaled;
+}
+
 #endif
