@@ -12,7 +12,8 @@
 #include "ykey.h"
 #include "yscrollbar.h"
 
-#include "yapp.h"
+#include "yxapp.h"
+#include "yprefs.h"
 #include "prefs.h"
 
 YColor *scrollBarBg(NULL);
@@ -206,7 +207,7 @@ void YScrollBar::getCoord(int &beg, int &end, int &min, int &max, int &nn) {
 
 // !!!! TODO: Warp3, Warp4, Motif borders
 
-void YScrollBar::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+void YScrollBar::paint(Graphics &g, const YRect &/*r*/) {
     int beg, end, min, max, nn;
 
     getCoord(beg, end, min, max, nn);
@@ -683,7 +684,7 @@ void YScrollBar::handleMotion(const XMotionEvent &motion) {
 
 bool YScrollBar::handleScrollKeys(const XKeyEvent &key) {
     if (key.type == KeyPress) {
-        KeySym k = XKeycodeToKeysym(app->display(), key.keycode, 0);
+        KeySym k = XKeycodeToKeysym(xapp->display(), key.keycode, 0);
         int m = KEY_MODMASK(key.state);
 
         switch (k) {
