@@ -9,13 +9,14 @@
 #ifdef CONFIG_TOOLTIP
 #include "base.h"
 #include "prefs.h"
+#include "yprefs.h"
 
 #include <string.h>
 
 YColor *YToolTip::toolTipBg = 0;
 YColor *YToolTip::toolTipFg = 0;
 
-YFont *YToolTip::toolTipFont = 0;
+ref<YFont> YToolTip::toolTipFont;
 YTimer *YToolTip::fToolTipVisibleTimer = 0;
 
 YToolTip::YToolTip(YWindow *aParent): YWindow(aParent) {
@@ -23,8 +24,8 @@ YToolTip::YToolTip(YWindow *aParent): YWindow(aParent) {
         toolTipBg = new YColor(clrToolTip);
     if (toolTipFg == 0)
         toolTipFg = new YColor(clrToolTipText);
-    if (toolTipFont == 0)
-        toolTipFont = YFont::getFont(toolTipFontName);
+    if (toolTipFont == null)
+        toolTipFont = YFont::getFont(XFA(toolTipFontName));
 
     fText = 0;
     setStyle(wsOverrideRedirect);
@@ -40,7 +41,7 @@ YToolTip::~YToolTip() {
     }
 }
 
-void YToolTip::paint(Graphics &g, int /*x*/, int /*y*/, unsigned int /*width*/, unsigned int /*height*/) {
+void YToolTip::paint(Graphics &g, const YRect &/*r*/) {
     g.setColor(toolTipBg);
     g.fillRect(0, 0, width(), height());
     g.setColor(YColor::black);
